#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>

#include "main.h"
#include "hcms.h"
#include "font5x7.h"

const unsigned int
	params_base_addr = 0x00,		//    
	names_base_addr = 0x10;			//     /

static unsigned char EEPROM_read(unsigned int addr)
{
	//     
	while (EECR & (1 << EEPE));
	EEARL = addr;
	EECR |= (1 << EERE);
	return EEDR;
}

static void EEPROM_update(unsigned char data, unsigned int addr)
{
	unsigned char temp;
	
	//     
	while (EECR & (1 << EEPE));
	EEARL = addr;
	EECR |= (1 << EERE);
	temp = EEDR;
	
	//       , 
	if (temp != data)
	{
		while (EECR & (1 << EEPE));
		EECR = (0 << EEPM1) | (0 >> EEPM0);
		EEARL = addr;
		EEDR = data;
		EECR |= (1 << EEMPE);
		EECR |= (1 << EEPE);
	}
}

void load_settings(void)
{
	unsigned char temp;
		
	//     
	temp = EEPROM_read(params_base_addr + 4);
	if ((temp > 0) & (temp <= MAX_MODULES))
	{
		modules_count = temp;
	}
	
	//     
	temp = EEPROM_read(params_base_addr + 5);
	if ((temp > 0) & (temp <= modules_count * RELAY_CNT - 1))
	{
		inputs_count = temp;
		outputs_count = modules_count * RELAY_CNT - inputs_count;
	}
	
	//    
	temp = EEPROM_read(params_base_addr);
	if ((temp >= 0) & (temp < inputs_count))
	{
		active_input = temp;
	}
	
	//    
	temp = EEPROM_read(params_base_addr + 1);
	if ((temp >= 0) & (temp < outputs_count))
	{
		active_output = temp;
	}
	
	//       /
	temp = EEPROM_read(params_base_addr + 7);
	if ((temp == IN_OUT) | (temp == OUT_IN))
	{
		in_out_order = temp;
	}
	
	//     
	temp = EEPROM_read(params_base_addr + 2);
	if ((temp == OFF) | (temp == ON))
	{
		lock_state = temp;
	}
	
	//     
	temp = EEPROM_read(params_base_addr + 3);
	if ((temp == OFF) | (temp == ON))
	{
		mute_state = temp;
	}
	
	//     
	temp = EEPROM_read(params_base_addr + 6);
	if ((temp > MIN_BRIGHTNESS) & (temp <= MAX_BRIGHTNESS))
	{
		hcms_bright(brightness = temp);
	}
	
	//   /  EEPROM
	for (unsigned int i = 0; i < 31 * NAME_WIDTH * 2; i++)
	{
		temp = EEPROM_read(names_base_addr + i);
		
		if (i < 31 * NAME_WIDTH)
		{
			//      
			if((temp < FONT_FIRST_CHAR) || (temp > FONT_LAST_CHAR))
			{
				//    
				i += NAME_WIDTH * 2 - 1;
			}
			else
			{
				//     
				input_names[i] = temp;
			}
		}
		else
		{
			if((temp < FONT_FIRST_CHAR) || (temp > FONT_LAST_CHAR))
			{
				i += NAME_WIDTH * 2 - 1;
			}
			else
			{
				output_names[i -  31 * NAME_WIDTH] = temp;
			}
		}
	}
}

void save_settings(void)
{
	// 0x00 -    
	EEPROM_update(active_input, params_base_addr);					//   EEPROM 
	EEPROM_update(active_output, params_base_addr + 1);				//   EEPROM 
	EEPROM_update(lock_state, params_base_addr + 2);				//   EEPROM   
	
	if (flag_save_mute == ON)
	{
		EEPROM_update(mute_state, params_base_addr + 3);			//   EEPROM   
	}
	
	EEPROM_update(modules_count, params_base_addr + 4);				//   EEPROM    
	EEPROM_update(inputs_count, params_base_addr + 5);				//   EEPROM    
	EEPROM_update(brightness, params_base_addr + 6);				//   EEPROM    
	EEPROM_update(in_out_order, params_base_addr + 7);				//   EEPROM     /
}