#include <avr/io.h>
#include <avr/pgmspace.h>

#include "main.h"
#include "hcms.h"
#include "font5x7.h"
#include "SPI.h"

char 
	hcms_screen[SCR_SIZE];					//  
					
static ctrl_reg0_t
	def_ctrl_r0;							//   0 
static ctrl_reg1_t							
	def_ctrl_r1;							//   1 

//  ,    -  
inline void hcms_reset(void)
{
	DISP_PORT_REG &= ~(1 << DISP_nRST_PIN);
	DISP_PORT_REG |= (1 << DISP_nRST_PIN);
}

//     (@param data -       )
static void hcms_ctrl_register(unsigned char data)
{
	hcms_ctrl_mode();
	hcms_enable();
	SPI_put_data(data, SCR_SIZE);
	hcms_disable();
}

//  "" 
void hcms_raw_pixels(unsigned char *buf, unsigned int sz)
{
	hcms_data_mode();
	hcms_enable();
	
	for (unsigned int i = 0; i < sz; i++)
	{
		SPI_put_data(buf[i], 8);
	}
	
	hcms_disable();
}

//     
static void rpad_strcpy(char *d, char *s)
{
	//   
	unsigned char length = 0;
	while(s[length] != '\0')
	{
		length++;
	}
	
	for(unsigned char i = 0; i < SCR_SIZE; i++)
	{
		if (i >= length)
		{
			d[i] = ' ';
		}
		else
		{
			d[i] = s[i];
		}
	}
}

//  
void hcms_puts(char *s)
{
	rpad_strcpy(hcms_screen, s);
	hcms_update_scr();
}

//        - @param c , * @return      
unsigned int get_offset_char(char c)
{
	if (c)
	{
		//     
		if((c < FONT_FIRST_CHAR) || (c > FONT_LAST_CHAR)) 
		{
			c = FONT_UNKNOWN_CHAR;
		}
	} 
	else 
	{
		//   '\0' -    
		c = ' ';
	}

	return (c - FONT_FIRST_CHAR) * FONT_COLS;
}

static void hcms_put_char(char c)
{
	unsigned int idx; //     

	//  
	idx  = get_offset_char(c);
	//  "" 
	for(unsigned char i = 0; i < FONT_COLS; i++)
	{
		SPI_put_data(pgm_read_byte(&font5x7[idx++]), 8);
	}
}

//     
void hcms_update_scr(void)
{
	hcms_data_mode();
	hcms_enable();

	for(unsigned char pos = 0; pos < SCR_SIZE; pos++)
	{
		hcms_put_char(hcms_screen[pos]);
	}
	
	hcms_disable();
}

//  
void hcms_bright(pwm_brightness_t br)
{
	def_ctrl_r0.brightness = br;
	hcms_ctrl_register(def_ctrl_r0.byte);
}

// - 
void hcms_on(uint8_t on)
{
	def_ctrl_r0.sleep_mode = !on;
	hcms_ctrl_register(def_ctrl_r0.byte);
}

//   
void hcms_peak_current(peak_current_t pc)
{
	def_ctrl_r0.peak_current = pc;
	hcms_ctrl_register(def_ctrl_r0.byte);
}

void hcms_init(void)
{
	//  
	hcms_reset();
	def_ctrl_r0.brightness = DEFAULT_BRIGHTNESS;
	def_ctrl_r0.peak_current = DEFAULT_PEAK_CURRENT;
	def_ctrl_r0.sleep_mode = 1;
	def_ctrl_r0.reg = 0;

	def_ctrl_r1.data_out_ctrl = 1;
	def_ctrl_r1.edo_prescaler = 0;
	def_ctrl_r1.reserved = 0;
	def_ctrl_r1.reg = 1;

	//  
	char scr_buff[SCR_SIZE];		//     ,   
		
	for (unsigned char i = 0; i < SCR_SIZE; i++)
	{
		scr_buff[i] = 0;
	}
	hcms_puts(scr_buff);
	// ---------
	
	hcms_ctrl_register(def_ctrl_r1.byte);
	hcms_ctrl_register(def_ctrl_r0.byte);
	
	hcms_bright(PWM_BRIGHT_037);
}