#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>

#include "main.h"
#include "hcms.h"
#include "font5x7.h"
#include "eeprom.h"
#include "SPI.h"

#define F_CPU 8000000UL					//   

//---------------------------------------------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------------------------------------------

unsigned char
	modules_count = 1,					//   
	inputs_count = 4,					//    
	outputs_count = 4,					//    
	setup_stage = SETUP_MODULES_STAGE,	//    
	operation_mode = MODE_STANDBY,		//    
	active_input = 0,					//  
	active_output = 0,					//  
	brightness = DEFAULT_BRIGHTNESS,	//   
	pwr_key_state = OPEN,				//    POWER
	in_key_state  = OPEN,				//    INPUT
	out_key_state = OPEN,				//    OUTPUT
	lock_key_state = OPEN,				//    LOCK (  INPUT)
	mute_key_state = OPEN,				//    MUTE (  OUTPUT)
	lock_state = OFF,					//       
	mute_state = ON,					//    MUTE
	in_out_order = OUT_IN,				// ,       - /  /
	allow_input_change = OFF;			//      (,          )

volatile unsigned char
	flag_scan_keys_short = 0,			//     
	flag_scan_keys_long = 0,			//     
	flag_relay_update = 0,				//    
	flag_hcms_update = 0,				//   
	flag_save_mute = ON;				//   MUTE  EEPROM

char
	input_names[31 * NAME_WIDTH]   = {" 01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31"}, //  
	output_names[31 * NAME_WIDTH] = {"01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 "};	//  

const char
	lock_symbols[2]						//   
		PROGMEM = {" @"},
	arrow_symbols[2]					//   
		PROGMEM = {":#"},
	numbers[10]							//  
		PROGMEM = {"0123456789"},
	brightness_value[15 * 2]			//    
		PROGMEM = {" 1 2 4 6 912151823303747608099"};

//---------------------------------------------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------------------------------------------

void init_system(void)
{
	//  
	SPI_DDR_REG |= (1 << SPI_SCK_PIN) | (1 << SPI_MOSI_PIN) | (1 << SPI_nDISP_PIN) | (1 << SPI_nREL_PIN);
	SPI_PORT_REG &= ~(1 << SPI_SCK_PIN);
	
	SPI_DDR_REG |= (1 << DISP_nRST_PIN) | (1 << DISP_RS_PIN);
	REL_DDR_REG |= (1 << REL_nRST_PIN);
	
	LED_DDR_REG |= (1 << LED_PWR_PIN);
	
	relays_disable();
	hcms_disable();
	
	//   0  Debouncer
	TIMSK0 |= (1 << TOIE0);										//     0  
	
	//   1   
	TCCR1B |= (1 << WGM12);										//    1 - CTC
	TIMSK1 |= (1 << OCIE1A);									//     1  
	OCR1A = DELAY_MS / 100 * 3125;								//    1,  DELAY_MS 
	
	//   Pin Change Interrupts	
	GIMSK  |= (1 << PCIE1) | (1 << PCIE0);						// Pin Change Interrupt Enable 0 and 1
	PCMSK0 |= (1 << PCINT7);									// Pin Change 7 Enable
	PCMSK1 |= (1 << PCINT9) | (1 << PCINT10);					// Pin Change 9 and 10 Enable
	
	//   
	set_sleep_mode(SLEEP_MODE_PWR_DOWN);						//    
	sleep_enable();												//    
	
	relays_reset();												//   
	hcms_init();												//  
	load_settings();											//    EEPROM
	LED_on();													//  
}

void display_update(void)
{
	char scr_buff[SCR_SIZE];		//     ,   
	
	//    STANDBY
	if (operation_mode == MODE_STANDBY)
	{
		//  
		for (unsigned char i = 0; i < 7; i++)
		{
			scr_buff[i] = 0;
		}
	}
	
	//    WORK
	else if (operation_mode == MODE_WORK)
	{
		//    "#XXX-YYY",
		//  # -   ,
		//  XXX -  
		//  - -    MUTE
		//  YYY -  
		scr_buff[0] = pgm_read_byte_near(lock_symbols + lock_state);
		
		for (unsigned char i = 0; i < 3; i++)
		{
			scr_buff[1 + i] = (input_names[(active_input) * NAME_WIDTH + i]);
		}
		
		scr_buff[4] = pgm_read_byte_near(arrow_symbols + mute_state);
		
		for (unsigned char i = 0; i < 3; i++)
		{
			scr_buff[5 + i] = (output_names[(active_output) * NAME_WIDTH + i]);
		}
	}
	
	//    SETUP
	else if (operation_mode == MODE_SETUP)
	{
		//   1  ,
		if (setup_stage == SETUP_MODULES_STAGE)
		{
			//    "RELMDL Y",  Y -    
			scr_buff[0] = pgm_read_byte_near(PSTR("R"));
			scr_buff[1] = pgm_read_byte_near(PSTR("E"));
			scr_buff[2] = pgm_read_byte_near(PSTR("L"));
			scr_buff[3] = pgm_read_byte_near(PSTR("M"));
			scr_buff[4] = pgm_read_byte_near(PSTR("D"));
			scr_buff[5] = pgm_read_byte_near(PSTR("L"));
			scr_buff[6] = pgm_read_byte_near(PSTR(" "));
			scr_buff[7] = pgm_read_byte_near(numbers + modules_count);
		}
		
		//   2  ,
		else if (setup_stage == SETUP_INOUTS_STAGE)
		{
			//    "IO XX-YY",  XX -  , YY -  
			scr_buff[0] = pgm_read_byte_near(PSTR("I"));
			scr_buff[1] = pgm_read_byte_near(PSTR("O"));
			scr_buff[2] = pgm_read_byte_near(PSTR(" "));
			scr_buff[3] = pgm_read_byte_near(numbers + inputs_count / 10);
			scr_buff[4] = pgm_read_byte_near(numbers + inputs_count % 10);
			scr_buff[5] = pgm_read_byte_near(PSTR("/"));
			scr_buff[6] = pgm_read_byte_near(numbers + outputs_count / 10);
			scr_buff[7] = pgm_read_byte_near(numbers + outputs_count % 10);
		}
		
		//   3  ,
		else if (setup_stage == SETUP_ORDER_STAGE)
		{
			//    "INs-OUTs"
			scr_buff[0] = pgm_read_byte_near(PSTR("O"));
			scr_buff[1] = pgm_read_byte_near(PSTR("R"));
			scr_buff[2] = pgm_read_byte_near(PSTR("D"));
			scr_buff[3] = pgm_read_byte_near(PSTR("R"));
			scr_buff[4] = pgm_read_byte_near(PSTR(" "));
			
			scr_buff[6] = pgm_read_byte_near(PSTR("_"));
			
			if (in_out_order == IN_OUT)
			{
				scr_buff[5] = pgm_read_byte_near(PSTR(">"));
				scr_buff[7] = pgm_read_byte_near(PSTR("<"));
			}
			else if (in_out_order == OUT_IN)
			{
				scr_buff[5] = pgm_read_byte_near(PSTR("<"));
				scr_buff[7] = pgm_read_byte_near(PSTR(">"));
			}
		}
		
		//   4  ,
		else if (setup_stage == SETUP_BRIGHT_STAGE)
		{
			//    "BRTN XX%",  XXX -   %
			scr_buff[0] = pgm_read_byte_near(PSTR("B"));
			scr_buff[1] = pgm_read_byte_near(PSTR("R"));
			scr_buff[2] = pgm_read_byte_near(PSTR("T"));
			scr_buff[3] = pgm_read_byte_near(PSTR("N"));
			scr_buff[4] = pgm_read_byte_near(PSTR(" "));
			
			for (unsigned char i = 0; i < 3; i++)
			{
				scr_buff[5 + i] = pgm_read_byte_near(brightness_value + (brightness - 1) * 2 + i);
			}
			
			scr_buff[7] = pgm_read_byte_near(PSTR("%"));
		}
		
		//       ,
		else if (setup_stage == SETUP_FIRMWARE_STAGE)
		{
			//    "VERS X.Y",  X -  , Y -  
			scr_buff[0] = pgm_read_byte_near(PSTR("V"));
			scr_buff[1] = pgm_read_byte_near(PSTR("E"));
			scr_buff[2] = pgm_read_byte_near(PSTR("R"));
			scr_buff[3] = pgm_read_byte_near(PSTR("S"));
			scr_buff[4] = pgm_read_byte_near(PSTR(" "));
			scr_buff[5] = pgm_read_byte_near(numbers + VERSION_MAJOR);
			scr_buff[6] = pgm_read_byte_near(PSTR("."));
			scr_buff[7] = pgm_read_byte_near(numbers + VERSION_MINOR);
		}
	}
	
	hcms_puts(scr_buff);												//    
}

inline void relays_reset(void)
{
	REL_PORT_REG &= ~(1 << REL_nRST_PIN);
	REL_PORT_REG |= (1 << REL_nRST_PIN);
	
	//        TPIC6B595
	relays_enable();
	relays_disable();
	
	//  ,     (5 )
	for (unsigned int i = 0; i < 5 * F_CPU / 1000; i++)
	{
		asm("nop");
	}
}

void relays_update(unsigned char input, unsigned char output, unsigned char mute)
{
	unsigned int temp = 0;
	
	//    
	//    "/"
	if (in_out_order == IN_OUT)
	{
		temp = ((1 << (input + outputs_count)) | (1 << output)) * (!mute & 0x01);
		//  mute      0b00000001,    
		//     MUTE temp   
	}
	//    "/"
	else if (in_out_order == OUT_IN)
	{
		temp = ((1 << (output + inputs_count)) | (1 << input)) * (!mute & 0x01);
	}	
	
	//   
	relays_reset();	
	
	//     SPI	
	SPI_put_data(temp, modules_count * RELAY_CNT);
	
	//        TPIC6B595
	relays_enable();
	relays_disable();
}

void scan_keys_short(void)
{
	//    POWER
	if (!(BTN_PWR_PIN_REG & (1 << BTN_PWR_PIN)))
	{
		if (pwr_key_state == OPEN)
		{
			set_to_zero_timer();					//  
			start_timer();							//  
			pwr_key_state = CLOSE;					//     POWER
		}
	}
	else if (BTN_PWR_PIN_REG & (1 << BTN_PWR_PIN))
	{
		if (pwr_key_state == CLOSE)
		{
			stop_timer();							//  
			sleep_enable();
			pwr_key_state = OPEN;					//     POWER
			
			//-----------------------------------------------------------------
			//     POWER
			//-----------------------------------------------------------------
			
			//    STANDBY
			if (operation_mode == MODE_STANDBY)
			{
				//      INPUT/OUTPUT,    
				if ((!(BTN_IN_PIN_REG & (1 << BTN_IN_PIN))) | (!(BTN_OUT_PIN_REG & (1 << BTN_OUT_PIN))))
				{
					operation_mode = MODE_SETUP;
					in_key_state = OPEN;
					out_key_state = OPEN;
				}
				//     
				else
				{
					operation_mode = MODE_WORK;
					LED_off();												//  
					load_settings();								//    EEPROM
					flag_save_mute = ON;
					flag_relay_update = ON;	
				}
			}
			
			//    WORK,    
			else if (operation_mode == MODE_WORK)
			{
				operation_mode = MODE_STANDBY;
				flag_save_mute = OFF;
				mute_state = ON;
				flag_relay_update = ON;
				LED_on();													//  
			}
			
			//    SETUP
			else if (operation_mode == MODE_SETUP)
			{
				//   1  ,   2 
				if (setup_stage == SETUP_MODULES_STAGE)
				{
					setup_stage = SETUP_INOUTS_STAGE;
				}
				//   2  ,   3 
				else if (setup_stage == SETUP_INOUTS_STAGE)
				{
					setup_stage = SETUP_ORDER_STAGE;
				}
				//   3  ,   4 
				else if (setup_stage == SETUP_ORDER_STAGE)
				{
					setup_stage = SETUP_BRIGHT_STAGE;
				}
				//   4        ,    
				else if ((setup_stage == SETUP_BRIGHT_STAGE) | (setup_stage == SETUP_FIRMWARE_STAGE))
				{
					operation_mode = MODE_WORK;
					setup_stage = SETUP_MODULES_STAGE;
					allow_input_change = OFF;
					flag_save_mute = ON;				
					LED_off();												//  								
				}
			}
		}
	}
	
	//    INPUT
	if (!(BTN_IN_PIN_REG & (1 << BTN_IN_PIN)))
	{
		if (in_key_state == OPEN)
		{
			set_to_zero_timer();					//  
			start_timer();							//  
			in_key_state = CLOSE;					//     INPUT
		}
	}
	else if (BTN_IN_PIN_REG & (1 << BTN_IN_PIN))
	{
		if (in_key_state == CLOSE)
		{
			stop_timer();							//  
			sleep_enable();
			in_key_state = OPEN;					//     INPUT
			
			//-----------------------------------------------------------------
			//     INPUT
			//-----------------------------------------------------------------
			//    WORK
			if (operation_mode == MODE_WORK)
			{
				//      
				if (lock_state == OFF)
				{
					//    
					if (active_input < (inputs_count - 1))
					{
						active_input++;
					}
					else
					{
						active_input = 0;
					}
					
					flag_relay_update = ON;
				}
			}
			
			//    SETUP
			else if (operation_mode == MODE_SETUP)
			{
				if (setup_stage == SETUP_MODULES_STAGE)
				{
					if (allow_input_change == OFF)
					{
						allow_input_change = ON;
					}
					else if (modules_count < MAX_MODULES)
					{
						modules_count++;
					}
					else
					{
						modules_count = 1;
					}
					
					if ((inputs_count > modules_count * RELAY_CNT / 2) | (outputs_count > modules_count * RELAY_CNT - inputs_count))
					{
						inputs_count = modules_count * RELAY_CNT / 2;
						outputs_count = modules_count * RELAY_CNT - inputs_count;
					}					
					
					//       , 
					//        
					if (active_input + 1 > inputs_count)
					{
						mute_state = ON;
						active_input = 0;
					}
					
					//       ,
					//        
					if (active_output + 1 > outputs_count)
					{
						mute_state = ON;
						active_output = 0;
					}
				}
				
				else if (setup_stage == SETUP_INOUTS_STAGE)
				{
					if (inputs_count < modules_count * RELAY_CNT - 1)
					{
						inputs_count++;
					}
					else
					{
						inputs_count = 1;
					}
					
					outputs_count = modules_count * RELAY_CNT - inputs_count;
					
					//       ,
					//        
					if (active_input + 1 > inputs_count)
					{
						mute_state = ON;
						active_input = 0;
					}
					
					//       ,
					//        
					if (active_output + 1 > outputs_count)
					{
						mute_state = ON;
						active_output = 0;
					}
				}
				
				else if (setup_stage == SETUP_ORDER_STAGE)
				{
					invert_state(in_out_order);
				}
				
				else if (setup_stage == SETUP_BRIGHT_STAGE)
				{
					if (brightness > MIN_BRIGHTNESS)
					{
						brightness--;
					}
					else
					{
						brightness = MAX_BRIGHTNESS;
					}
					hcms_bright(brightness);
				}
			}
		}
	}
	
	//    OUTPUT
	if (!(BTN_OUT_PIN_REG & (1 << BTN_OUT_PIN)))
	{
		if (out_key_state == OPEN)
		{
			set_to_zero_timer();					//  
			start_timer();							//  
			out_key_state = CLOSE;					//     OUTPUT
		}
	}
	else if (BTN_OUT_PIN_REG & (1 << BTN_OUT_PIN))
	{
		if (out_key_state == CLOSE)
		{
			stop_timer();							//  
			sleep_enable();
			out_key_state = OPEN;					//     OUTPUT
			
			//-----------------------------------------------------------------
			//     OUTPUT
			//-----------------------------------------------------------------
			//    WORK
			if (operation_mode == MODE_WORK)
			{
				//      
				if (lock_state == OFF)
				{
					//    
					if (active_output < (outputs_count - 1))
					{
						active_output++;
					}
					else
					{
						active_output = 0;
					}
					
					flag_relay_update = ON;
				}
			}
			
			//    SETUP
			else if (operation_mode == MODE_SETUP)
			{
				if (setup_stage == SETUP_MODULES_STAGE)
				{
					if (allow_input_change == OFF)
					{
						allow_input_change = ON;
					}
					
					else if (modules_count > 1)
					{
						modules_count--;						
					}
					else
					{
						modules_count = MAX_MODULES;
					}
					
					if ((inputs_count > modules_count * RELAY_CNT / 2) | (outputs_count > modules_count * RELAY_CNT - inputs_count))
					{
						inputs_count = modules_count * RELAY_CNT / 2;
						outputs_count = modules_count * RELAY_CNT - inputs_count;
					}
					
					//       ,
					//        
					if (active_input + 1 > inputs_count)
					{
						mute_state = ON;
						active_input = 0;
					}
					
					//       ,
					//        
					if (active_output + 1 > outputs_count)
					{
						mute_state = ON;
						active_output = 0;
					}
				}
				
				else if (setup_stage == SETUP_INOUTS_STAGE)
				{
					if (outputs_count < modules_count * RELAY_CNT - 1)
					{
						outputs_count++;
					}
					else
					{
						outputs_count = 1;
					}
					
					inputs_count = modules_count * RELAY_CNT - outputs_count;
					
					//       ,
					//        
					if (active_input + 1 > inputs_count)
					{
						mute_state = ON;
						active_input = 0;
					}
					
					//       ,
					//        
					if (active_output + 1 > outputs_count)
					{
						mute_state = ON;
						active_output = 0;
					}
				}
				
				else if (setup_stage == SETUP_ORDER_STAGE)
				{
					invert_state(in_out_order);
				}
				
				else if (setup_stage == SETUP_BRIGHT_STAGE)
				{
					if (brightness < MAX_BRIGHTNESS)
					{
						brightness++;
					}
					else
					{
						brightness = MIN_BRIGHTNESS;
					}
					hcms_bright(brightness);
				}
			}
		}
	}
	
	flag_hcms_update = ON;
	save_settings();
}

void scan_keys_long(void)
{
	//-----------------------------------------------------------------
	//     INPUT  OUTPUT
	//-----------------------------------------------------------------
	if ((in_key_state == CLOSE) & (out_key_state == CLOSE))
	{
		in_key_state = OPEN;
		out_key_state = OPEN;
		
		//    WORK
		if (operation_mode == MODE_WORK)
		{
			//      /
			setup_stage = SETUP_INOUTS_STAGE;
			operation_mode = MODE_SETUP;
			mute_state = ON;
		}
		
		//    SETUP
		else if (operation_mode == MODE_SETUP)
		{
			//          ,
			//       
			operation_mode = MODE_WORK;
			setup_stage = SETUP_MODULES_STAGE;
			LED_off();												//  
		}
		
		flag_relay_update = ON;
	}
	
	//-----------------------------------------------------------------
	//     POWER
	//-----------------------------------------------------------------
	if (pwr_key_state == CLOSE)
	{
		pwr_key_state = OPEN;
		
		//    WORK
		if (operation_mode == MODE_SETUP)
		{
			//       
			setup_stage = SETUP_FIRMWARE_STAGE;
		}
		
		flag_relay_update = ON;
	}
	
	//-----------------------------------------------------------------
	//     INPUT
	//-----------------------------------------------------------------
	if (in_key_state == CLOSE)
	{
		in_key_state = OPEN;
		
		//    WORK
		if (operation_mode == MODE_WORK)
		{
			invert_state(lock_state);		//   
			
		}
	}
	
	//-----------------------------------------------------------------
	//     OUTPUT
	//-----------------------------------------------------------------
	else if (out_key_state == CLOSE)
	{
		out_key_state = OPEN;
		
		//    WORK
		if (operation_mode == MODE_WORK)
		{
			invert_state(mute_state);		//   MUTE
			
		}
		
		flag_relay_update = ON;
	}
	
	flag_hcms_update = ON;
	save_settings();
}

//---------------------------------------------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------------------------------------------
ISR(PCINT0_vect)
{
	sleep_disable();				//   
	start_debouncer();				//  ,   
}

ISR(PCINT1_vect)
{
	sleep_disable();				//   
	start_debouncer();				//  ,   
}	

ISR(TIM0_OVF_vect)
{
	stop_debouncer();				//  ,   , ...
	set_to_zero_debouncer();		// ...   
	flag_scan_keys_short = ON;		//    
}

ISR(TIM1_COMPA_vect)
{
	stop_timer();					//  ,   , ...
	set_to_zero_timer();			// ...   
	flag_scan_keys_long = ON;		//    
}

//---------------------------------------------------------------------------------------------------------------
//  MAIN 
//---------------------------------------------------------------------------------------------------------------
int main(void)
{
	init_system();													// 
	sei();															//   

	while (1) 
    {
		//        
		if (flag_scan_keys_short)
		{
			flag_scan_keys_short = OFF;
			scan_keys_short();										//   
		}
		
		//        
		if (flag_scan_keys_long)
		{
			flag_scan_keys_long = OFF;
			scan_keys_long();										//   
		}
	
		//      
		if (flag_relay_update)
		{
			flag_relay_update = OFF;
			relays_update(active_input, active_output, mute_state);	//   
		}
		
		//     
		if (flag_hcms_update)
		{
			flag_hcms_update = OFF;
			display_update();										//  
		}		
		
		sleep_cpu();												//    
    }
}