#ifndef MAIN_H_
#define MAIN_H_

//---------------------------------------------------------------------------------------------------------------
//   -
//---------------------------------------------------------------------------------------------------------------
// 
#define LED_DDR_REG		DDRB
#define LED_PWR_PIN		0

// 
#define BTN_PWR_PIN_REG	PINB
#define BTN_PWR_PIN		2
#define BTN_IN_PIN_REG	PINB
#define BTN_IN_PIN		1
#define BTN_OUT_PIN_REG	PINA
#define BTN_OUT_PIN		7

// SPI
#define SPI_PORT_REG	PORTA
#define SPI_DDR_REG		DDRA
#define SPI_SCK_PIN		4
#define SPI_MOSI_PIN	6

#define REL_PORT_REG	PORTA
#define REL_DDR_REG		DDRA
#define REL_nRST_PIN	5

#define SPI_nDISP_PIN	1
#define SPI_nREL_PIN	2

// 
#define DISP_PORT_REG	PORTA
#define DISP_PIN_REG	PINA
#define DISP_DDR_REG	DDRA
#define DISP_nRST_PIN	0				// Display Reset
#define DISP_RS_PIN		3				// Display Register Select (Dot Register (RS = logic low) or Control Register (RS = logic high))

//---------------------------------------------------------------------------------------------------------------
// 
//---------------------------------------------------------------------------------------------------------------
#define start_timer()				TCCR1B |=  (1 << CS12)					//   
#define stop_timer()				TCCR1B &= ~(1 << CS12)					//   
#define set_to_zero_timer()			TCNT1 = 0								//   

#define start_debouncer()			TCCR0B |=   (1 << CS02) | (1 << CS00)	//    
#define stop_debouncer()			TCCR0B &= ~((1 << CS02) | (1 << CS00))	//    
#define set_to_zero_debouncer();	TCNT0 = 0								//    

#define LED_on()					PORTB |=  (1 << LED_PWR_PIN)			//   
#define LED_off()					PORTB &= ~(1 << LED_PWR_PIN)			//   

#define hcms_ctrl_mode()			DISP_PORT_REG |=  (1 << DISP_RS_PIN)	//       
#define hcms_data_mode()			DISP_PORT_REG &= ~(1 << DISP_RS_PIN)	//     

#define hcms_enable()				SPI_PORT_REG &= ~(1 << SPI_nDISP_PIN)	//   
#define hcms_disable()				SPI_PORT_REG |=  (1 << SPI_nDISP_PIN)	//   

#define relays_enable()				SPI_PORT_REG &= ~(1 << SPI_nREL_PIN)	//   
#define relays_disable()			SPI_PORT_REG |=  (1 << SPI_nREL_PIN)	//    

#define spi_clk_hi()				SPI_PORT_REG |=  (1 << SPI_SCK_PIN)		//       CLK
#define spi_clk_lo()				SPI_PORT_REG &= ~(1 << SPI_SCK_PIN)		//       CLK
#define spi_do_hi()					SPI_PORT_REG |=  (1 << SPI_MOSI_PIN)	//       
#define spi_do_lo()					SPI_PORT_REG &= ~(1 << SPI_MOSI_PIN)	//       

#define invert_state(x)				x ^= (1 << 0)							//  

//---------------------------------------------------------------------------------------------------------------
//  
//---------------------------------------------------------------------------------------------------------------

//   
#define VERSION_MAJOR			2
#define VERSION_MINOR			0

#define DELAY_MS				800		//      (0..2000)
#define MAX_MODULES				4		//     
#define RELAY_CNT				8		//      

//  
#define MODE_STANDBY			0		//  
#define MODE_WORK				1		//  
#define MODE_SETUP				2		//  

//  
#define CLOSE					0		//  
#define OPEN					1		//  

//  
#define OFF						0		// 
#define ON						1		// 

//  
#define SETUP_MODULES_STAGE		1		//     
#define SETUP_INOUTS_STAGE		2		//    /
#define SETUP_ORDER_STAGE		3		//      - /  /
#define SETUP_BRIGHT_STAGE		4		//    /
#define SETUP_FIRMWARE_STAGE	5		//     

//   /  
#define IN_OUT					0		// /
#define OUT_IN					1		// /

#define NAME_WIDTH				3		//   /

extern unsigned char
	modules_count,						//   
	inputs_count,						//    
	outputs_count,						//    
	setup_stage,						//    
	operation_mode,						//    
	active_input,						//  
	active_output,						//  
	brightness,							//   
	pwr_key_state,						//    POWER
	in_key_state,						//    INPUT
	out_key_state,						//    OUTPUT
	lock_key_state,						//    LOCK (  INPUT)
	mute_key_state,						//    MUTE (  OUTPUT)
	lock_state,							//       
	mute_state,							//    MUTE
	in_out_order;						// ,       - /  /

extern volatile unsigned char
	flag_save_mute;						//   MUTE  EEPROM
	
extern char
	input_names[31 * NAME_WIDTH],
	output_names[31 * NAME_WIDTH];

//  
void init_system(void);

//  
void display_update(void);

//      
void relays_reset(void);

//   
void relays_update(unsigned char input, unsigned char output, unsigned char mute);

//     
void scan_keys_short(void);

//     
void scan_keys_long(void);

#endif /* MAIN_H_ */